@@ -8,6 +8,7 @@ Defines base classes
 import functools
 import operator
 import logging
+import json
 from pathlib import Path 
 from abc import ABCMeta, abstractmethod, abstractproperty
 from typing import (Tuple, Callable, Optional, Union, Any, Dict, TypeVar,
@@ -48,11 +49,12 @@ class FieldBase(metaclass=ABCMeta):
             Name of the field
     """ 
     
-    _subclasses: Dict[str, Any] = {}  # all classes inheriting from this
+    _subclasses: Dict[str, 'FieldBase'] = {}  # all classes inheriting from this
     readonly = False
     
     
-    def __init__(self, grid: GridBase, data: OptionalArrayLike = None,
+    def __init__(self, grid: GridBase,
+                 data: OptionalArrayLike = None,
                  label: Optional[str] = None):
         """ 
         Args:
@@ -76,32 +78,27 @@ class FieldBase(metaclass=ABCMeta):
 
 
     @classmethod
-    def from_state(cls, state: Union[str, Dict[str, Any]],
-                   grid: GridBase, data=None) -> "FieldBase":
+    def from_state(cls, attributes: Dict[str, Any],
+                   data: np.ndarray = None) -> "FieldBase":
         """ create a field from given state.
         
         Args:
-            state (str or dict): State from which the instance is created. If 
-                `state` is a string, it is decoded as JSON.
-            grid (:class:`~pde.grids.GridBase`):
-                The grid that is used to describe the field
-            data (:class:`numpy.ndarray`, optional): Data values at the support
-                points of the grid that define the field.
+            attributes (dict):
+                The attributes that describe the current instance
+            data (:class:`numpy.ndarray`, optional):
+                Data values at the support points of the grid defining the field
         """
-        # decode the json data
-        if isinstance(state, str):
-            import json
-            state = dict(json.loads(state))
-        
-        # create the instance of the correct class
-        class_name = state.pop('class')
+        # base class was chosen => select correct class from attributes
+        class_name = attributes.pop('class')
+
         if class_name == cls.__name__:
             raise RuntimeError('Cannot reconstruct abstract class' 
                                f'`{class_name}`')
-        field_cls = cls._subclasses[class_name]
-        return field_cls.from_state(state, grid, data=data)  # type: ignore
+        
+        # call possibly overwritten classmethod from subclass
+        return cls._subclasses[class_name].from_state(attributes, data)
 
-    
+            
     @classmethod
     def from_file(cls, filename: str) -> "FieldBase":
         """ create field by reading file
@@ -115,7 +112,7 @@ class FieldBase(metaclass=ABCMeta):
         with h5py.File(filename, "r") as fp:
             if 'class' in fp.attrs:
                 # this should be a field collection
-                assert fp.attrs['class'] == 'FieldCollection'
+                assert json.loads(fp.attrs['class']) == 'FieldCollection'
                 obj = FieldCollection._from_hdf_dataset(fp)
                 
             elif len(fp) == 1:
@@ -132,12 +129,16 @@ class FieldBase(metaclass=ABCMeta):
     @classmethod
     def _from_hdf_dataset(cls, dataset) -> "FieldBase":
         """ construct a field by reading data from an hdf5 dataset """
-        class_name = dataset.attrs['class']
+        # copy attributes from hdf
+        attributes = dict(dataset.attrs)
+        
+        # determine class
+        class_name = json.loads(attributes.pop('class'))
         field_cls = cls._subclasses[class_name]
         
-        grid = GridBase.from_state(dataset.attrs['grid'])
-        label = dataset.attrs['label'] if 'label' in dataset.attrs else None
-        return field_cls(grid, data=dataset, label=label)  # type: ignore
+        # unserialize the attributes
+        attributes = field_cls.unserialize_attributes(attributes)
+        return field_cls.from_state(attributes, data=dataset)
 
     
     @property
@@ -175,13 +176,14 @@ class FieldBase(metaclass=ABCMeta):
                              f'extensions `{extension}`')
 
 
-    def _write_hdf_dataset(self, fp, key: str = 'data'):
-        """ write data to a given hdf5 file pointer `fp` """
-        dataset = fp.create_dataset(key, data=self.data)
-        dataset.attrs['class'] = self.__class__.__name__
-        if self.label:      
-            dataset.attrs['label'] = str(self.label)      
-        dataset.attrs['grid'] = self.grid.state_serialized      
+    def _write_hdf_dataset(self, hdf_path, key: str = 'data'):
+        """ write data to a given hdf5 path `hdf_path` """
+        # write the data
+        dataset = hdf_path.create_dataset(key, data=self.data)
+
+        # write attributes        
+        for key, value in self.attributes_serialized.items():
+            dataset.attrs[key] = value
 
 
     def _write_to_image(self, filename: str, **kwargs):
@@ -239,19 +241,47 @@ class FieldBase(metaclass=ABCMeta):
         
         
     @property
-    def state(self) -> dict:
-        """ dict: current state of this instance """
-        return {'label': self.label}
+    def attributes(self) -> Dict[str, Any]:
+        """ dict: describes the state of the instance (without the data) """
+        return {'class': self.__class__.__name__,
+                'grid': self.grid,
+                'label': self.label}
     
-        
+
     @property
-    def state_serialized(self) -> str:
-        """ str: a json serialized version of the field """
-        import json
-        state = self.state
-        state['class'] = self.__class__.__name__
-        return json.dumps(state)
+    def attributes_serialized(self) -> Dict[str, str]:
+        """ dict: serialized version of the attributes """
+        results = {}
+        for key, value in self.attributes.items():
+            if key == 'grid':
+                results[key] = value.state_serialized
+            else:
+                results[key] = json.dumps(value)
+        return results
     
+    
+    @classmethod
+    def unserialize_attributes(cls, attributes: Dict[str, str]) \
+            -> Dict[str, Any]:
+        """ unserializes the given attributes
+        
+        Args:
+            attributes (dict):
+                The serialized attributes
+                
+        Returns:
+            dict: The unserialized attributes
+        """
+        # base class was chosen => select correct class from attributes
+        class_name = json.loads(attributes['class'])
+
+        if class_name == cls.__name__:
+            raise RuntimeError('Cannot reconstruct abstract class' 
+                               f'`{class_name}`')
+        
+        # call possibly overwritten classmethod from subclass
+        return cls._subclasses[class_name].unserialize_attributes(attributes)
+
 
     @property
     def _data_flat(self):
@@ -482,111 +512,6 @@ class FieldBase(metaclass=ABCMeta):
         """
         raise NotImplementedError()
     
-    
-    def plot_line(self, **kwargs):
-        r""" visualize a field using a 1d cut """
-        raise NotImplementedError()
-    
-    
-    def plot_image(self, ax=None,
-                   colorbar: bool = False,
-                   transpose: bool = False,
-                   title: Optional[str] = None, **kwargs):
-        r""" visualize an 2d image of the field
-
-        Args:
-            ax: Figure axes to be used for plotting. If `None`, a new figure is
-                created
-            colorbar (bool): determines whether a colorbar is shown
-            transpose (bool): determines whether the transpose of the data
-                should be plotted.
-            title (str): Title of the plot. If omitted, the title is chosen
-                automatically based on the label the data field.
-            \**kwargs: Additional keyword arguments are passed to
-                `matplotlib.pyplot.imshow`.
-                
-        Returns:
-            Result of `plt.imshow`
-        """
-        raise NotImplementedError()
-    
-    
-    def plot_vector(self,
-                    method: str = 'quiver',
-                    ax=None,
-                    transpose: bool = False,
-                    title: Optional[str] = None,
-                    show: bool = False,
-                    **kwargs):
-        r""" visualize a 2d vector field
-
-        Args:
-            method (str): Plot type that is used. This can be either `quiver`
-                or `streamplot`.
-            ax: Figure axes to be used for plotting. If `None`, a new figure is
-                created
-            transpose (bool): determines whether the transpose of the data
-                should be plotted.
-            title (str): Title of the plot. If omitted, the title is chosen
-                automatically based on the label the data field.
-            show (bool):
-                Flag setting whether :func:`matplotlib.pyplot.show` is called
-            \**kwargs: Additional keyword arguments are passed to
-                :func:`matplotlib.pyplot.quiver` or
-                :func:`matplotlib.pyplot.streamplot`.
-                
-        Returns:
-            Result of `plt.quiver` or `plt.streamplot`
-        """
-        raise NotImplementedError()
-    
-            
-    def plot(self, kind: str = 'auto', filename=None, **kwargs):
-        r""" visualize the field
-        
-        Args:
-            kind (str): Determines the visualizations. Supported values are
-                `image`,  `line`, or `vector`. Alternatively, `auto` determines
-                the best visualization based on the field itself.
-            filename (str, optional): If given, the plot is written to the
-                specified file. Otherwise, the plot might show directly in an
-                interactive matplotlib session or `matplotlib.pyplot.show()`
-                might be used to display the graphics.
-            ax: Figure axes to be used for plotting. If `None`, a new figure is
-                created
-            \**kwargs: All additional keyword arguments are forwarded to the
-                actual plotting functions.
-                
-        Returns:
-            The result of the respective matplotlib plotting function
-        """
-        if kind == 'auto':
-            # determine best plot for this field
-            if (isinstance(self, DataFieldBase) and self.rank == 1 and
-                    self.grid.dim == 2):
-                kind = 'vector'
-            elif len(self.grid.shape) == 1:
-                kind = 'line'
-            else:
-                kind = 'image'
-
-        # do the actual plotting
-        if kind == 'image':
-            res = self.plot_image(**kwargs)
-        elif kind == 'line':
-            res = self.plot_line(**kwargs)
-        elif kind == 'vector':
-            res = self.plot_vector(**kwargs)
-        else:
-            raise ValueError(f'Unsupported plot `{kind}`. Possible choices are '
-                             '`image`, `line`, `vector`, or `auto`.')
-            
-        # store the result to a file if requested
-        if filename:
-            plt.savefig(filename)
-            
-        return res
-                    
 
 
 TDataField = TypeVar('TDataField', bound='DataFieldBase')
@@ -832,22 +757,25 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
          
         return cls(grid, data, label=label)
     
-        
+
     @classmethod
-    def from_state(cls, state: Dict[str, Any], grid: GridBase,  # type: ignore
-                   data=None) -> "DataFieldBase":
+    def from_state(cls, attributes: Dict[str, Any],
+                   data: np.ndarray = None) -> "DataFieldBase":
         """ create a field from given state.
         
         Args:
-            state (str or dict): State from which the instance is created. If 
-                `state` is a string, it is decoded as JSON.
-            grid (:class:`~pde.grids.GridBase`):
-                The grid that is used to describe the field
-            data (:class:`numpy.ndarray`, optional): Data values at the support
-                points of the grid that define the field.
+            attributes (dict):
+                The attributes that describe the current instance
+            data (:class:`numpy.ndarray`, optional):
+                Data values at the support points of the grid defining the field
         """
-        return cls(grid, data=data, **state)
+        if 'class' in attributes:
+            class_name = attributes.pop('class')
+            assert class_name == cls.__name__
             
+        # create the instance from the attributes
+        return cls(attributes.pop('grid'), data=data, **attributes)
+        
         
     def copy(self: TDataField, data=None, label: str = None) -> TDataField:
         """ return a copy of the data, but not of the grid
@@ -871,6 +799,27 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
         return (self.grid.dim,) * self.rank
     
     
+    @classmethod
+    def unserialize_attributes(cls, attributes: Dict[str, str]) \
+            -> Dict[str, Any]:
+        """ unserializes the given attributes
+        
+        Args:
+            attributes (dict):
+                The serialized attributes
+                
+        Returns:
+            dict: The unserialized attributes
+        """
+        results = {}
+        for key, value in attributes.items():
+            if key == 'grid':
+                results[key] = GridBase.from_state(value)
+            else:
+                results[key] = json.loads(value)
+        return results
+    
+        
     def _write_to_image(self, filename: str, **kwargs):
         r""" write data to image 
         
@@ -989,7 +938,7 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
         
         Args:
             bc:
-                {ARG_BOUNDARIES}
+                The boundary conditions applied to the field. {ARG_BOUNDARIES}
             fill (float, optional):
                 Determines how values out of bounds are handled. If `None`, a
                 `ValueError` is raised when out-of-bounds points are requested.
@@ -1047,8 +996,8 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
             # turn points into a linear array
             point = np.atleast_1d(point)
             if point.shape[-1] != grid_dim:
-                raise ValueError('Invalid points for interpolation.'
-                                 '(Dimension mismatch)')
+                raise DimensionError('Dimension of the interpolation point '
+                                     'does not match grid dimension')
             points_shape = point.shape[:-1]  # keep original shape of points
                 
             num_points = functools.reduce(operator.mul, points_shape, 1)
@@ -1349,25 +1298,28 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
             
             
     def plot_line(self, 
+                  ax=None,
                   extract: str = 'auto',
                   ylabel: str = None,
-                  ax=None,
                   title: str = None,
+                  filename: str = None,
                   show: bool = False,
                   **kwargs):
-        r""" visualize a field using a 1d cut
+        r""" visualize a field using a 1d line plot
         
         Args:
+            ax:
+                Figure axes to be used for plotting. If `None`, a new figure is
+                created
             extract (str):
                 The method used for extracting the line data.
             ylabel (str):
                 Label of the y-axis. If omitted, the label is chosen 
                 automatically from the data field.
-            ax:
-                Figure axes to be used for plotting. If `None`, a new figure is
-                created
             title (str):
                 Title of the plot.
+            filename (str, optional):
+                If given, the plot is written to the specified file.
             show (bool):
                 Flag setting whether :func:`matplotlib.pyplot.show` is called
             \**kwargs:
@@ -1395,7 +1347,7 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
             ax.set_ylabel(ylabel)
             
         from ..visualization.plotting import finalize_plot
-        finalize_plot(ax, title=title, show=show)
+        finalize_plot(ax, title=title, filename=filename, show=show)
             
         return line
     
@@ -1404,9 +1356,10 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
                    colorbar: bool = False,
                    transpose: bool = False,
                    title: Optional[str] = None,
+                   filename: str = None,
                    show: bool = False,
                    **kwargs):
-        r""" visualize an 2d image of the field
+        r""" visualize a field using a 2d density plot
 
         Args:
             ax:
@@ -1419,6 +1372,8 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
             title (str):
                 Title of the plot. If omitted, the title is chosen automatically
                 based on the label the data field.
+            filename (str, optional):
+                If given, the plot is written to the specified file.
             show (bool):
                 Flag setting whether :func:`matplotlib.pyplot.show` is called
             \**kwargs:
@@ -1438,44 +1393,46 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
         for key in ['performance_goal', 'scalar']:
             if key in kwargs:
                 get_image_args[key] = kwargs.pop(key)
-        img = self.get_image_data(**get_image_args)
+        data = self.get_image_data(**get_image_args)
         
         if transpose:
             # adjust image data such that the transpose is plotted
-            img['data'] = img['data'].T
-            img['label_x'], img['label_y'] = img['label_y'], img['label_x']
+            data['data'] = data['data'].T
+            data['label_x'], data['label_y'] = data['label_y'], data['label_x']
         
         # plot the image (either using pyplot or the supplied axes)
         kwargs.setdefault('origin', 'lower')
         kwargs.setdefault('interpolation', 'none')
         if ax is None:
             ax = plt.figure().gca()
-        res = ax.imshow(img['data'], extent=img['extent'], **kwargs)
+        img = ax.imshow(data['data'], extent=data['extent'], **kwargs)
             
         # set some default properties
-        ax.set_xlabel(img['label_x'])
-        ax.set_ylabel(img['label_y'])
+        ax.set_xlabel(data['label_x'])
+        ax.set_ylabel(data['label_y'])
         if title is None:
-            title = img.get('title', self.label)
+            title = data.get('title', self.label)
 
         if colorbar:
             from ..tools.misc import add_scaled_colorbar
-            add_scaled_colorbar(res, ax=ax)
+            add_scaled_colorbar(img, ax=ax)
 
         from ..visualization.plotting import finalize_plot
-        finalize_plot(ax, title=title, show=show)
+        finalize_plot(ax, title=title, filename=filename, show=show)
             
-        return res
+        return img
 
 
     def plot_vector(self,
                     method: str = 'quiver',
                     ax=None,
                     transpose: bool = False,
+                    max_points: int = 16,
                     title: Optional[str] = None,
+                    filename: str = None,
                     show: bool = False,
                     **kwargs):
-        r""" visualize a 2d vector field
+        r""" visualize a field using a 2d vector plot
 
         Args:
             method (str):
@@ -1486,9 +1443,14 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
                 created
             transpose (bool):
                 Determines whether the transpose of the data should be plotted.
+            max_points (int):
+                The maximal number of points that is used along each axis. This
+                is only used for quiver plots.
             title (str):
                 Title of the plot. If omitted, the title is chosen
                 automatically based on the label the data field.
+            filename (str, optional):
+                If given, the plot is written to the specified file.
             show (bool):
                 Flag setting whether :func:`matplotlib.pyplot.show` is called
             \**kwargs:
@@ -1501,12 +1463,7 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
             :func:`matplotlib.pyplot.streamplot`
         """
         # obtain image data
-        get_image_args = {}
-        # FIXME: rename scalar_method to method
-        for key in ['performance_goal', 'scalar_method']:
-            if key in kwargs:
-                get_image_args[key] = kwargs.pop(key)
-        data = self.get_vector_data(**get_image_args)
+        data = self.get_vector_data()
         
         if transpose:
             # adjust image data such that the transpose is plotted
@@ -1516,6 +1473,19 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
         
         # determine which plotting function to use
         if method == 'quiver':
+            # potentially reduce the number of points along each axis
+            shape = data['data_x'].shape
+            for axis, size in enumerate(shape):
+                if size > max_points:
+                    idx_f = np.linspace(0, size - 1, max_points)
+                    idx_i = np.round(idx_f).astype(int)
+                    #
+                    data['data_x'] = np.take(data['data_x'], idx_i, axis=axis)
+                    data['data_y'] = np.take(data['data_y'], idx_i, axis=axis)
+                    if axis == 0:
+                        data['y'] = data['y'][idx_i]
+                    elif axis == 1:
+                        data['x'] = data['x'][idx_i]
             plot_func = plt.quiver if ax is None else ax.quiver
             
         elif method == 'streamplot':
@@ -1538,6 +1508,51 @@ class DataFieldBase(FieldBase, metaclass=ABCMeta):
             title = data.get('title', self.label)
 
         from ..visualization.plotting import finalize_plot
-        finalize_plot(ax, title=title, show=show)
+        finalize_plot(ax, title=title, filename=filename, show=show)
             
         return res
+
+
+    def plot(self, kind: str = 'auto', **kwargs):
+        r""" visualize the field
+        
+        Args:
+            kind (str):
+                Determines the visualizations. Supported values are `image`, 
+                `line`, or `vector`. Alternatively, `auto` determines the best
+                visualization based on the field itself.
+            \**kwargs:
+                All additional keyword arguments are forwarded to the actual
+                plotting functions. This includes `ax` to determine the
+                matplotlib axis in which the plot is done, `title` to set the
+                title of the plot, `filename` to write the plot to a specific
+                file, and `show` to determine whether
+                :func:`matplotlib.pyplot.show` is called. These parameters are
+                described in detail in :meth:`~DataFieldBase.plot_line`.
+                
+        Returns:
+            The result of the respective matplotlib plotting function
+        """
+        if kind == 'auto':
+            # determine best plot for this field
+            if (isinstance(self, DataFieldBase) and self.rank == 1 and
+                    self.grid.dim == 2):
+                kind = 'vector'
+            elif len(self.grid.shape) == 1:
+                kind = 'line'
+            else:
+                kind = 'image'
+
+        # do the actual plotting
+        if kind == 'image':
+            res = self.plot_image(**kwargs)
+        elif kind == 'line':
+            res = self.plot_line(**kwargs)
+        elif kind == 'vector':
+            res = self.plot_vector(**kwargs)
+        else:
+            raise ValueError(f'Unsupported plot `{kind}`. Possible choices are '
+                             '`image`, `line`, `vector`, or `auto`.')
+            
+        return res
+    
\ No newline at end of file
