@@ -38,10 +38,14 @@ class ScalarField(DataFieldBase):
         
         
     @classmethod
+    @fill_in_docstring
     def from_expression(cls, grid: GridBase, expression: str,
                         label: str = None) -> "ScalarField":
         """ create a scalar field on a grid from a given expression
         
+        Warning:
+            {WARNING_EXEC}
+        
         Args:
             grid (:class:`~pde.grids.GridBase`):
                 Grid defining the space on which this field is defined
@@ -56,7 +60,9 @@ class ScalarField(DataFieldBase):
         expr = ScalarExpression(expression=expression, signature=grid.axes)
         points = {name: grid.cell_coords[..., i]
                   for i, name in enumerate(grid.axes)}
-        return cls(grid=grid, data=expr(**points), label=label)
+        return cls(grid=grid,  # lgtm [py/call-to-non-callable]
+                   data=expr(**points),
+                   label=label)
     
     
     @classmethod
@@ -114,6 +120,7 @@ class ScalarField(DataFieldBase):
         
         Args:
             bc:
+                The boundary conditions applied to the field.
                 {ARG_BOUNDARIES}
             out (ScalarField, optional):
                 Optional scalar field to which the  result is written.
@@ -137,6 +144,7 @@ class ScalarField(DataFieldBase):
         
         Args:
             bc: 
+                The boundary conditions applied to the field.
                 {ARG_BOUNDARIES}
             out (VectorField, optional):
                 Optional vector field to which the result is written.
