@@ -9,7 +9,6 @@ import numpy as np
 
 from .. import FileStorage
 from ...grids import UnitGrid
-from ...grids.base import GridBase
 from ...fields import ScalarField
 from ...pdes import DiffusionPDE
 from ...tools.misc import skipUnlessModule
@@ -90,7 +89,7 @@ def test_simulation_persistence(compression):
     np.testing.assert_almost_equal(storage.times, [0, 0.05, 0.1])
     data = np.array(storage.data)
     assert data.shape == (3,) + state.data.shape
-    grid_res = GridBase.from_state(storage.info['grid'])
+    grid_res = storage.grid
     assert grid == grid_res
     grid_res = storage.grid
     assert grid == grid_res
