@@ -14,6 +14,7 @@ Functions and classes for plotting simulation data
 
 import logging
 import warnings
+import time
 from typing import (Union, Callable, Optional, Any, Dict, List, Tuple)
 
 import numpy as np
@@ -48,7 +49,7 @@ def finalize_plot(fig_or_ax=None,
                   title: str = None,
                   filename: str = None,
                   show: bool = False,
-                  close_figure: bool = False):
+                  close_figure: bool = False) -> Tuple[Any, Any]:
     r""" finalizes a figure by adjusting relevant parameters
     
     Args:
@@ -65,7 +66,7 @@ def finalize_plot(fig_or_ax=None,
             Whether the figure should be closed in the end
             
     Returns:
-        Result of :func:`matplotlib.pyplot.imshow`
+        tuple: The figure and the axes that were used to finalize the plot
     """
     import matplotlib.pyplot as plt
     
@@ -93,7 +94,7 @@ def finalize_plot(fig_or_ax=None,
     if close_figure:
         plt.close(fig)
         
-    return fig
+    return fig, ax
     
 
 
@@ -142,12 +143,21 @@ def extract_field(fields: FieldBase,
 
 
 class ScalarFieldPlot():
-    """ class managing compound plots of scalar fields """
+    """ class managing compound plots of scalar fields
+    
+    Note:
+        The `ScalarFieldPlot` class supports jupyter notebooks and can update
+        plots dynamically. However, the current implementation deletes all other
+        output in the same cell when `show = True`.    
+    """
+    
     
     @fill_in_docstring
     def __init__(self, fields: FieldBase,
                  quantities=None,
                  scale: ScaleData = 'automatic',
+                 title: Optional[str] = None,
+                 tight: bool = True,
                  show: bool = True):
         """
         Args:
@@ -157,10 +167,14 @@ class ScalarFieldPlot():
                 {ARG_PLOT_QUANTITIES}
             scale (str, float, tuple of float):
                 {ARG_PLOT_SCALE}
+            title (str):
+                Title of the plot.
+            tight (bool):
+                Whether to call :func:`matplotlib.pyploy.tight_layout`
             show (bool):
                 Flag determining whether to show a plot. If `False`, the plot is
                 kept in the background, which can be useful if it only needs to
-                be written to a file
+                be written to a file.
         """
         import matplotlib.pyplot as plt
         import matplotlib.cm as cm
@@ -174,11 +188,21 @@ class ScalarFieldPlot():
         num_rows = len(self.quantities)
         num_cols = max(len(p) for p in self.quantities)
              
+        # initialize display to update ipython output dynamically
+        try:
+            from IPython import display
+        except ImportError:
+            self._ipython_display = None
+        else:
+            self._ipython_display = display
+                 
         # set up the figure
         self.fig, self.axes = plt.subplots(num_rows, num_cols,
                                            sharey=True, squeeze=False,
                                            figsize=(4 * num_cols, 4 * num_rows))
-        self.sup_title = plt.suptitle('')
+        if title is None:
+            title = ""        
+        self.sup_title = plt.suptitle(title)
         
         # set up all images
         empty = np.zeros_like(example_image['data'])
@@ -205,13 +229,13 @@ class ScalarFieldPlot():
                 cmap = panel.get('cmap')
                 if cmap is None:
                     if vmin is None or vmax is None:
-                        cmap = cm.PiYG
+                        cmap = cm.viridis
                     elif np.isclose(-vmin, vmax):
                         cmap = cm.coolwarm
                     elif np.isclose(vmin, 0):
                         cmap = cm.gray
                     else:
-                        cmap = cm.PiYG
+                        cmap = cm.viridis
 
                 # initialize image    
                 ax = self.axes[i, j]
@@ -228,8 +252,13 @@ class ScalarFieldPlot():
                 img_row.append(img)
             self.images.append(img_row)
             
-        self.show_data(fields)    
+        if tight:
+            # adjust layout and leave some room for title
+            self.fig.tight_layout(rect=[0, 0.03, 1, 0.95])
         
+        # show the actual data
+        self.show_data(fields, first_frame=True)    
+
     
     @classmethod
     @fill_in_docstring
@@ -333,14 +362,19 @@ class ScalarFieldPlot():
         self.fig.savefig(path, **kwargs)
         
             
-    def show_data(self, fields: FieldBase, title: Optional[str] = None) -> None:
+    def show_data(self, fields: FieldBase, title: Optional[str] = None,
+                  first_frame: bool = False) -> None:
         """ show the given fields in the current view
         
         Args:
-            fields: The field or field collection of which the defined 
-                quantities are shown.
-            title (str, optional): The title of this view. If `None`, the
-                current title is not changed. 
+            fields:
+                The field or field collection of which the defined quantities
+                are shown.
+            title (str, optional):
+                The title of this view. If `None`, the current title is not
+                changed. 
+            first_frame (bool):
+                Indicates whether this is the first frame
         """
         import matplotlib.pyplot as plt
         assert isinstance(fields, FieldBase)
@@ -348,6 +382,7 @@ class ScalarFieldPlot():
         if title:
             self.sup_title.set_text(title)
         
+        # iterate over all panels and update their content
         for i, panel_row in enumerate(self.quantities):
             for j, panel in enumerate(panel_row):
                 # obtain image data
@@ -364,11 +399,22 @@ class ScalarFieldPlot():
                     vmax = img_data['data'].max()
                 img.set_clim(vmin, vmax)
                 
-        # add a small pause to allow the GUI to run it's event loop
+        # display the updated panels if selected
         if self.show:
-            with warnings.catch_warnings():
-                warnings.simplefilter("ignore")
-                plt.pause(0.01)
+            if self._ipython_display:
+                # seems to be in an ipython instance => update widget
+                if not first_frame:
+                    self._ipython_display.clear_output(wait=True)
+                self._ipython_display.display(self.fig)
+                # add a small pause to allow the GUI to run it's event loop
+                time.sleep(0.01)
+                
+            else:
+                # seems to be in a normal matplotlib window => update it
+                with warnings.catch_warnings():
+                    warnings.simplefilter("ignore")
+                    # add a small pause to allow the GUI to run it's event loop
+                    plt.pause(0.01)
                 
     
     def make_movie(self, storage: StorageBase,
@@ -379,10 +425,12 @@ class ScalarFieldPlot():
         Args:
             storage (:class:`~pde.storage.base.StorageBase`):
                 The storage instance that contains all the data for the movie
-            filename (str): The filename to which the movie is written. The
-                extension determines the format used.
-            progress (bool): Flag determining whether the progress of making
-                the movie is shown.
+            filename (str):
+                The filename to which the movie is written. The extension
+                determines the format used.
+            progress (bool):
+                Flag determining whether the progress of making the movie is
+                shown.
         """
         from ..visualization.movies import Movie
 
@@ -405,6 +453,7 @@ def plot_magnitudes(storage: StorageBase,
                     filename: str = None,
                     quantities=None,
                     ax=None,
+                    title: str = None,
                     show: bool = False,
                     **kwargs) -> None:
     r""" create a plot of spatially integrated quantities from a given storage.
@@ -423,6 +472,8 @@ def plot_magnitudes(storage: StorageBase,
         ax:
             Matplotlib figure axes to be used for plotting. If `None`, a new
             figure is created
+        title (str):
+            Determines the title of the figure
         show (bool):
             Flag determining whether :func:`matplotlib.pyplot.show` is called
         \**kwargs:
@@ -467,7 +518,7 @@ def plot_magnitudes(storage: StorageBase,
             value *= quantity.get('scale', 1)
             data[i]['values'].append(value)
     
-    close_figure = (filename and ax is None)
+    close_figure = (bool(filename) and ax is None)
     if ax is None:
         # create new figure
         ax = plt.figure().gca()
@@ -477,16 +528,15 @@ def plot_magnitudes(storage: StorageBase,
         kwargs['label'] = d['label']
         ax.plot(storage.times, d['values'], **kwargs)
         
+    ax.set_xlabel('Time')
+    if len(data) == 1:
+        ax.set_ylabel(kwargs['label'])
+        
     if len(data) > 1:
         ax.legend(loc='best')
     
-    fig = ax.figure
-    if filename:
-        fig.savefig(filename)
-    if show:
-        plt.show()
-    if close_figure:
-        plt.close(fig)
+    finalize_plot(ax, title=title, filename=filename, show=show,
+                  close_figure=close_figure)
             
 
 
@@ -495,6 +545,7 @@ def plot_kymograph(storage: StorageBase,
                    extract: str = 'auto',
                    colorbar: bool = True,
                    transpose: bool = False,
+                   title: str = None,
                    show: bool = False,
                    ax=None,
                    **kwargs):
@@ -519,6 +570,8 @@ def plot_kymograph(storage: StorageBase,
             Whether to show a colorbar or not
         transpose (bool):
             Determines whether the transpose of the data should is plotted
+        title (str):
+            Determines the title of the figure
         show (bool):
             Flag determining whether :func:`matplotlib.pyplot.show` is called
         ax: Figure axes to be used for plotting. If `None`, a new figure is
@@ -560,8 +613,11 @@ def plot_kymograph(storage: StorageBase,
     if colorbar:
         from ..tools.misc import add_scaled_colorbar
         add_scaled_colorbar(res, ax=ax)
+        
+    if title is None:
+        title = img_data['label_y']
 
-    finalize_plot(ax, title=img_data['label_y'], filename=filename, show=show,
+    finalize_plot(ax, title=title, filename=filename, show=show,
                   close_figure=close_figure)
     
     return res
